#include <xc.h>
#include "qei.h"
#include "sw_config.h"

void qeiInit(void)
{
    QEI1IOCbits.HOMPOL = 1; // home signal is active low
    // zero out the position counter
    POS1HLD = 0;
    POS1CNTL = 0;
    QEI1CONbits.QEIEN = 1; // enable QEI module
    // since homing on index in hardware doesn't seem to work with the ultra
    // short index pulses of the sensor, we use an external interrupt:
    IPC5bits.INT1IP = 5;
}

void qeiSetHomingMode(void)
{
    QEI1ICH = 0;
    QEI1ICL = 0;
    QEI1CONbits.PIMOD = 3; // index when home is active resets position counter
}

void qeiSetSimpleHomingMode(void)
{
    // enable interrupt
    IFS1bits.INT1IF = 0;
    IEC1bits.INT1IE = 1;
    
    QEI1ICH = 0;
    QEI1ICL = 0;
    // also set hardware homing mode, even though it will not work without index pin assigned
    QEI1CONbits.IMV = 0;//2;
    QEI1CONbits.PIMOD = 2; // index resets position counter
    
}

unsigned int qeiCheckHomingMode(void)
{
    return (unsigned int) QEI1CONbits.PIMOD;
}

void qeiSetNormalMode(void)
{
    QEI1CONbits.PIMOD = 0; // index input is counted, but has no effect
}

long qeiReadPosition(void)
{
    union
    {
        long pos_all;
        struct
        {
            int pos_low;
            int pos_high;
        };
    } position;
    
    position.pos_low = POS1CNTL; // read low word first, this stores the high
    // word in the hold register:
    position.pos_high = POS1HLD; // reads the upper word
    return position.pos_all;
}

void qeiWritePosition(long position)
{
    POS1HLD = (int) (position >> 16);
    POS1CNTL = (int) position;
}

int qeiReadSpeed(void)
{
    return VEL1CNT;
}

void __attribute__((__interrupt__, __auto_psv__)) _INT1Interrupt(void)
{
    qeiWritePosition(0);
    QEI1CONbits.PIMOD = 0; // set qei back to normal mode
    IFS1bits.INT1IF = 0;
    IEC1bits.INT1IE = 0;
}